from deepclas4bio import Evaluator
from deepclas4bio.PredictorFactory import PredictorFactory
import importlib

def evaluate(readDataset,path,pathLabels,measures,predictors):

    classReadDataset__ = getattr(importlib.import_module('deepclas4bio.'+readDataset), readDataset)
    rd=classReadDataset__()

    evaluator = Evaluator.Evaluator(rd, path,
                                    pathLabels)

    # Anadimos las medidas
    for measure in measures:
        evaluator.addMeasure(measure)

    # Anadimos los constructores
    m = PredictorFactory()
    for predictor in predictors:

        model = m.getPredictor(predictor['framework'], predictor['model'])
        evaluator.addPredictor(model)

    return evaluator.evaluate()



from deepclas4bio import DatasetManager
from deepclas4bio.PredictorFactory import PredictorFactory

def predict(image,framework,model):
    predictor_factory=PredictorFactory()
    modelo=predictor_factory.getPredictor(framework,model)
    return modelo.predict(image)

def predictBatch(images,framework,model,batch=64):
    predictor_factory=PredictorFactory()
    modelo=predictor_factory.getPredictor(framework,model)
    dataManager= DatasetManager.DatasetManager(images, batch=batch)
    predictions=[]
    while(dataManager.hasNextBach()):
        batchImages=dataManager.nextBatch()
        prediction=modelo.predictBatch(batchImages)
        predictions+=prediction
    results=[]
    for p in predictions:
        results.append(modelo.model.postProcessor(p))
    return results


import sys
import json
from deepclas4bio.predict import predict as pred
from deepclas4bio.predict import predictBatch as predBatch
from deepclas4bio.listMethods import listFrameworks as lsFrameworks
from deepclas4bio.listMethods import listModels as lsModels
from deepclas4bio.listMethods import listMeasures as lsMeasures
from deepclas4bio.evaluate import evaluate as ev
from deepclas4bio.listMethods import listReadDatasets as lsDataset

def predict():

    image = sys.argv[1]
    framework=sys.argv[2]
    model=sys.argv[3]

    result = pred(image, framework, model)

    data = {}
    data['type'] = 'classification'
    data['image'] = image
    data['framework'] = framework
    data['model'] = model
    data['class'] = result
    with open('data.json', 'w') as f:
        json.dump(data, f)


def predictBatch():

    with open(sys.argv[1]) as file:
        data = json.load(file)

    framework = data['framework']
    model = data['model']
    images = data['images']

    results = predBatch(images, framework, model)

    resultImages = []
    for image, result in zip(images, results):
        object = {}
        object['image'] = image
        object['class'] = result
        resultImages.append(object)

    data = {}
    data['type'] = 'classification'
    data['framework'] = framework
    data['model'] = model
    data['results'] = resultImages
    with open('data.json', 'w') as f:
        json.dump(data, f)


def listFrameworks():
    data = {}
    data['type'] = 'frameworks'
    data['frameworks'] = lsFrameworks()
    with open('data.json', 'w') as f:
        json.dump(data, f)


def listModels():
    framework=sys.argv[1]
    data = {}
    data['type'] = 'models'
    data['framework'] = framework
    data['models'] = lsModels(framework)
    with open('data.json', 'w') as f:
        json.dump(data, f)

def listMeasures():
    data = {}
    data['type'] = 'measures'
    data['measures'] = lsMeasures()
    with open('data.json', 'w') as f:
        json.dump(data, f)

def evaluate():
    config=sys.argv[1]

    with open(config) as file:
        data=json.load(file)


    readDataset=data['readDataset']
    path = data['pathDataset']
    pathLabels = data['pathLabels']
    measures = data['measures']
    predictors = data['predictors']

    result=ev(readDataset,path,pathLabels,measures,predictors)
    with open('data.json', 'w') as f:
        json.dump(result, f)

def listReadDatasets():
    data = {}
    data['type'] = 'readDatasets'
    data['readDatasets'] = lsDataset()
    with open('data.json', 'w') as f:
        json.dump(data, f)




from deepclas4bio import Predictor
from subprocess import *
import os
from pathlib import Path
import requests

class DL4JPredictor(Predictor.Predictor):

    def jarWrapper(self,*args):
        process=Popen(['java', '-jar']+list(args),stdout=PIPE,stderr=PIPE)
        result=[]

        for line in process.stdout:
            result.append(line.decode('utf-8').rstrip('\n').rstrip('\n'))
        return result

    def downloadJar(self):
        path = str(Path.home()) + os.sep + 'DeepClas4BioModels' + os.sep + 'DL4J'
        if not os.path.exists(path + os.sep + 'PredictDL4J.jar'):
            if not os.path.exists(path):
                os.makedirs(path)
            r = requests.get(
                'https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/EYHxT1EyIeRJluozEP2YlbIBSd7TIVAyNbfCGPm-gqySyg?download=1')
            with open(path + os.sep + 'PredictDL4J.jar', 'wb') as f:
                f.write(r.content)


    def predict(self,image):
        self.downloadJar()
        path=str(Path.home()) + os.sep + 'DeepClas4BioModels' + os.sep + 'DL4J'
        path=path+os.sep+'PredictDL4J.jar'

        args=[path,self.model.name[:-4],image]
        result=self.jarWrapper(*args)
        return result[0]

    def predictBatch(self,images):
        self.downloadJar()
        path = str(Path.home()) + os.sep + 'DeepClas4BioModels' + os.sep + 'DL4J'
        path = path + os.sep + 'PredictDL4J.jar'

        args = [path, self.model.name[:-4]]+images
        result = self.jarWrapper(*args)
        newResult=[]
        for r in result:
            r=r.replace("[","").replace("]","")
            newResult.append(list(map(int,map(float,r.split(",")))))
        return newResult



from deepclas4bio.listMethods import *
from deepclas4bio.predict import *
from deepclas4bio.evaluate import *

__all__=[predict,predictBatch,listFrameworks,listModels,listMeasures,listReadDatasets,evaluate]

predict.__module__='deepclas4bio'
predictBatch.__module__='deepclas4bio'
listFrameworks.__module__='deepclas4bio'
listModels.__module__='deepclas4bio'
listMeasures.__module__='deepclas4bio'
listReadDatasets.__module__='deepclas4bio'
evaluate.__module__='deepclas4bio'



import numpy as np
import os
import requests
from pathlib import Path

# Add your model here
models=['VGG16','VGG19','ResNet50','GoogLeNet']

def vgg16dl4jpreprocess(im):
    pass

def vgg19dl4jpreprocess(im):
    pass

def resnet50dl4jpreprocess(im):
    pass

def googlenetdl4jpreprocess(im):
    pass



def vgg16dl4jload():
    pass

def vgg19dl4jload():
    pass

def resnet50dl4jload():
    pass

def googlenetdl4jload():
    pass


def commonPostProcess(result):
    path = str(Path.home()) + os.sep + 'DeepClas4BioModels'
    if not os.path.exists(path + os.sep + 'synset_words.txt'):
        if not os.path.exists(path):
            os.makedirs(path)
        r = requests.get(
            'https://unirioja-my.sharepoint.com/:t:/g/personal/adines_unirioja_es/ERS2ZWkLvc1AqY8FqIEjKBQB8MMobadwzrWsw4g86DBdAg?download=1')
        with open(path + os.sep + 'synset_words.txt', 'wb') as f:
            f.write(r.content)
    labels = np.loadtxt(path +os.sep+ "synset_words.txt", str, delimiter='\n')
    result=np.asarray(result)
    return labels[result.argmax()]

def vgg16dl4jpostprocess(result):
    return commonPostProcess(result)

def vgg19dl4jpostprocess(result):
    return commonPostProcess(result)

def resnet50dl4jpostprocess(result):
    return commonPostProcess(result)

def googlenetdl4jpostprocess(result):
    return commonPostProcess(result)



from threading import Thread
from deepclas4bio.PredictorFactory import PredictorFactory
import flask
import redis
import uuid
import time
import json
import os


IMAGE_WIDTH = 224
IMAGE_HEIGHT = 224
IMAGE_CHANS = 3
IMAGE_DTYPE = "float32"


IMAGE_QUEUE = "image_queue"
BATCH_SIZE = 32
SERVER_SLEEP = 0.25
CLIENT_SLEEP = 0.25


app = flask.Flask(__name__)
app_root=os.path.dirname(os.path.abspath(__file__))
db = redis.StrictRedis(host="localhost", port=6379, db=0)
model = None
framework="Keras"
modelName="VGG16"



def classify_process():
    print("* Loading model...")
    predictor_factory=PredictorFactory()
    model = predictor_factory.getPredictor(framework, modelName)
    print("* Model loaded")

    while True:
        queue = db.lrange(IMAGE_QUEUE, 0, BATCH_SIZE - 1)
        imageIDs = []
        batch = []

        for q in queue:
            q = json.loads(q.decode("utf-8"))

            image=q['image']

            batch.append(image)


            imageIDs.append(q["id"])

        if len(imageIDs) > 0:

            for (imageID, imageName) in zip(imageIDs, batch):

                results = model.predict(imageName)
                db.set(imageID, json.dumps(results))

            db.ltrim(IMAGE_QUEUE, len(imageIDs), -1)

        time.sleep(SERVER_SLEEP)



@app.route("/predict", methods=["POST"])
def predict():
    data = {"success": False}
    if flask.request.method == "POST":
        if flask.request.files.get("image"):
            k = str(uuid.uuid4())
            image = flask.request.files["image"]
            destination = os.path.join(app_root, 'temp/' + k + image.filename)

            image.save(destination)

            d = {"id": k, "image": destination}
            db.rpush(IMAGE_QUEUE, json.dumps(d))

            while True:
                output = db.get(k)

                if output is not None:
                    output = output.decode("utf-8")
                    data['type'] = 'classification'
                    data['image'] = image.filename
                    data['framework'] = framework
                    data['model'] = modelName
                    data['class'] = json.loads(output)
                    data['success'] = True

                    db.delete(k)
                    os.remove(destination)
                    break
                time.sleep(CLIENT_SLEEP)
    return flask.jsonify(data)


if __name__ == "__main__":

    print("* Starting model service...")
    t = Thread(target=classify_process, args=())
    t.daemon = True
    t.start()

    print("* Starting web service...")
    app.run()


import os
import numpy as np
import mxnet as mx
from mxnet import gluon,nd
from mxnet.gluon.model_zoo import vision
from pathlib import Path
import requests
import skimage.io as io

# Add your model here
models=['VGG11','VGG13','VGG16','VGG19','DenseNet121','DenseNet161','DenseNet169','DenseNet201','InceptionV3','AlexNet',
        'ResNet18','ResNet34','ResNet50','ResNet101','SqueezeNet','MobileNet','ResNet34Kvasir']



######## METHODS FOR LOAD MODELS ########

# Generic method to laod models from name
def loadModel(modelName):
    path = str(Path.home()) + os.sep + 'DeepClas4BioModels' + os.sep + 'MxNet' + os.sep
    pathModel=path+'Classification'+os.sep+'model'+os.sep+modelName+'.json'
    pathWeights=path+'Classification'+os.sep+'weights'+os.sep+modelName+'.params'


    ctx = mx.gpu() if mx.test_utils.list_gpus() else mx.cpu()
    deserialized_net=gluon.nn.SymbolBlock.imports(pathModel, ['data'], pathWeights, ctx=ctx)

    return deserialized_net

def vgg11mxnetload():
    net=vision.vgg11(pretrained=True)
    net.hybridize()
    return net

def vgg13mxnetload():
    net=vision.vgg13(pretrained=True)
    net.hybridize()
    return net

def vgg16mxnetload():
    net= vision.vgg16(pretrained=True)
    net.hybridize()
    return net

def vgg19mxnetload():
    net=vision.vgg19(pretrained=True)
    net.hybridize()
    return net

def densenet121mxnetload():
    net = vision.densenet121(pretrained=True)
    net.hybridize()
    return net

def densenet161mxnetload():
    net = vision.densenet161(pretrained=True)
    net.hybridize()
    return net

def densenet169mxnetload():
    net = vision.densenet169(pretrained=True)
    net.hybridize()
    return net

def densenet201mxnetload():
    net = vision.densenet201(pretrained=True)
    net.hybridize()
    return net


def inceptionv3mxnetload():
    net = vision.inception_v3(pretrained=True)
    net.hybridize()
    return net

def alexnetmxnetload():
    net = vision.alexnet(pretrained=True)
    net.hybridize()
    return net

def resnet18mxnetload():
    net = vision.resnet18_v2(pretrained=True)
    net.hybridize()
    return net

def resnet34mxnetload():
    net = vision.resnet34_v2(pretrained=True)
    net.hybridize()
    return net

def resnet50mxnetload():
    net = vision.resnet50_v2(pretrained=True)
    net.hybridize()
    return net

def resnet101mxnetload():
    net = vision.resnet101_v2(pretrained=True)
    net.hybridize()
    return net

def resnet152mxnetload():
    net = vision.resnet152_v2(pretrained=True)
    net.hybridize()
    return net

def squeezenetmxnetload():
    net = vision.squeezenet1_1(pretrained=True)
    net.hybridize()
    return net


def mobilenetmxnetload():
    net = vision.mobilenet_v2_1_0(pretrained=True)
    net.hybridize()
    return net


def resnet34kvasirmxnetload():
    net=loadModel('ResNet34Kvasir')
    net.hybridize()
    return net



######## METHODS FOR PREPROCESS ########
def commonPreProcess(im):
    img=mx.nd.array(io.imread(im)).astype(np.uint8)
    img=mx.image.resize_short(img,256)
    img,_=mx.image.center_crop(img,(224,224))
    img=mx.image.color_normalize(img.astype(np.float32)/255,
                                 mean=mx.nd.array([0.485,0.456,0.406]),
                                 std=mx.nd.array([0.229,0.224,0.225]))
    img=mx.nd.transpose(img.astype('float32'),(2,1,0))
    return img

def vgg11mxnetpreprocess(im):
    return commonPreProcess(im)

def vgg13mxnetpreprocess(im):
    return commonPreProcess(im)

def vgg16mxnetpreprocess(im):
    return commonPreProcess(im)

def vgg19mxnetpreprocess(im):
    return commonPreProcess(im)

def densenet121mxnetpreprocess(im):
    return commonPreProcess(im)

def densenet161mxnetpreprocess(im):
    return commonPreProcess(im)

def densenet169mxnetpreprocess(im):
    return commonPreProcess(im)

def densenet201mxnetpreprocess(im):
    return commonPreProcess(im)

def inceptionv3mxnetpreprocess(im):
    return commonPreProcess(im)

def alexnetmxnetpreprocess(im):
    return commonPreProcess(im)

def resnet18mxnetpreprocess(im):
    return commonPreProcess(im)

def resnet34mxnetpreprocess(im):
    return commonPreProcess(im)

def resnet50mxnetpreprocess(im):
    return commonPreProcess(im)

def resnet101mxnetpreprocess(im):
    return commonPreProcess(im)

def resnet152mxnetpreprocess(im):
    return commonPreProcess(im)

def squeezenetmxnetpreprocess(im):
    return commonPreProcess(im)

def mobilenetmxnetpreprocess(im):
    return commonPreProcess(im)

def resnet34kvasirmxnetpreprocess(im):
    img = mx.nd.array(io.imread(im)).astype(np.uint8)
    img=mx.image.resize_short(img,227)
    img,_=mx.image.center_crop(img,(227,227))
    #     mg=mx.image.color_normalize(img.astype(np.float32)/255,
    #                                  mean=mx.nd.array([0.485,0.456,0.406]),
    #                                  std=mx.nd.array([0.229,0.224,0.225]))
    img = mx.nd.transpose(img.astype('float32'), (2, 1, 0))
    # img = mx.nd.expand_dims(img, axis=0)
    return img

######## METHODS FOR POSPROCESS ########
def commonPostProcess(result):
    result=np.squeeze(result)
    result=np.argsort(result)[::-1]
    path = str(Path.home()) + os.sep + 'DeepClas4BioModels'
    if not os.path.exists(path + os.sep + 'synset_words.txt'):
        if not os.path.exists(path):
            os.makedirs(path)
        r = requests.get(
            'https://unirioja-my.sharepoint.com/:t:/g/personal/adines_unirioja_es/ERS2ZWkLvc1AqY8FqIEjKBQB8MMobadwzrWsw4g86DBdAg?download=1')
        with open(path + os.sep + 'synset_words.txt', 'wb') as f:
            f.write(r.content)
    labels=np.loadtxt(path+os.sep+"synset_words.txt",str,delimiter='\n')
    return labels[result[0]]


def vgg11mxnetpostprocess(result):
    return commonPostProcess(result)

def vgg13mxnetpostprocess(result):
    return commonPostProcess(result)

def vgg16mxnetpostprocess(result):
    return commonPostProcess(result)

def vgg19mxnetpostprocess(result):
    return commonPostProcess(result)

def densenet121mxnetpostrocess(result):
    return commonPostProcess(result)

def densenet161mxnetpostrocess(result):
    return commonPostProcess(result)

def densenet169mxnetpostrocess(result):
    return commonPostProcess(result)

def densenet201mxnetpostrocess(result):
    return commonPostProcess(result)

def inceptionv3mxnetpostprocess(result):
    return commonPostProcess(result)

def alexnetmxnetpostprocess(result):
    return commonPostProcess(result)

def resnet18mxnetpostprocess(result):
    return commonPostProcess(result)

def resnet34mxnetpostprocess(result):
    return commonPostProcess(result)

def resnet50mxnetpostprocess(result):
    return commonPostProcess(result)

def resnet101mxnetpostprocess(result):
    return commonPostProcess(result)

def resnet152mxnetpostprocess(result):
    return commonPostProcess(result)

def squeezenetmxnetpostprocess(result):
    return commonPostProcess(result)

def mobilenetmxnetpostprocess(result):
    return commonPostProcess(result)

def resnet34kvasirmxnetpostprocess(result):
    result=np.squeeze(result)
    # result=result.asnumpy()
    result=np.ndarray.argsort(result)[::-1]
    labels=['dyed-lifted-polyps', 'dyed-resection-margins', 'esophagitis', 'normal-cecum', 'normal-pylorus', 'normal-z-line', 'polyps', 'ulcerative-colitis']
    return labels[result[0]]


import importlib

class ModelFactory:
    loadedModels={}
    def getModel(self,framework,model):
        modelName=framework+model
        m=self.loadedModels.get(modelName)
        if m is not None:
            return m
        else:
            class_name=framework+'Model'
            class__=getattr(importlib.import_module('deepclas4bio.'+framework+'.'+class_name),class_name)
            loadModel=model+framework+'load'
            loadModel=loadModel.lower()
            loadModelMethod = getattr(importlib.import_module('deepclas4bio.'+framework + '.' + framework + 'Functions'), loadModel)
            preprocess=model+framework+'preprocess'
            preprocess=preprocess.lower()
            preprocessMethod=getattr(importlib.import_module('deepclas4bio.'+framework+'.'+framework+'Functions'),preprocess)
            postprocess = model + framework + 'postprocess'
            postprocess = postprocess.lower()
            postprocessMethod = getattr(importlib.import_module('deepclas4bio.'+framework + '.' + framework + 'Functions'), postprocess)
            m= class__(model+framework,loadModelMethod,preprocessMethod,postprocessMethod)
            self.loadedModels[modelName]=m
            return m



import os
import numpy as np
import torch
from PIL import Image
import importlib
from torchvision import transforms
from torch.autograd import Variable
import torchvision.models as torchmodels
from pathlib import Path
import sys
import requests

# Add your model here
models=['VGG11','VGG13','VGG16','VGG19','AlexNet','DenseNet121','DenseNet161','DenseNet169','DenseNet201','InceptionV3',
        'ResNet18','ResNet34','ResNet50','ResNet101','ResNet152','SqueezeNet10','SqueezeNet11','ResNet34Kvasir']



######## METHODS FOR LOAD MODELS ########

# Generic method to laod models from name
def loadModel(modelName):
    sys.path.append(os.path.abspath(str(Path.home()) + os.sep + 'DeepClas4BioModels' + os.sep + 'PyTorch' + os.sep+'Classification' + os.sep + 'model'))
    class_name=modelName
    class__=getattr(importlib.import_module(class_name),class_name)
    modelo=class__()

    path = str(Path.home()) + os.sep + 'DeepClas4BioModels' + os.sep + 'PyTorch' + os.sep


    pathWeights ='Classification' + os.sep + 'weights' + os.sep + modelName + '.pth'
    pathWeights=path+pathWeights
    state=torch.load(pathWeights,torch.device('cpu'))
    modelo.load_state_dict(state_dict=state, strict=False)
    modelo=modelo.eval()
    return modelo


def vgg11pytorchload():
    model= torchmodels.vgg11(pretrained=True)
    return model.eval()

def vgg13pytorchload():
    model= torchmodels.vgg13(pretrained=True)
    return model.eval()

def vgg16pytorchload():
    model= torchmodels.vgg16(pretrained=True)
    return model.eval()

def vgg19pytorchload():
    return torchmodels.vgg19(pretrained=True)

def alexnetpytorchload():
    model= torchmodels.alexnet(pretrained=True)
    return model.eval()

def densenet121pytorchload():
    model= torchmodels.densenet121(pretrained=True)
    return model.eval()

def densenet161pytorchload():
    model= torchmodels.densenet161(pretrained=True)
    return model.eval()

def densenet169pytorchload():
    model= torchmodels.densenet169(pretrained=True)
    return model.eval()

def densenet201pytorchload():
    model= torchmodels.densenet201(pretrained=True)
    return model.eval()

def inceptionv3pytorchload():
    model= torchmodels.inception_v3(pretrained=True)
    return model.eval()

def resnet18pytorchload():
    model= torchmodels.resnet18(pretrained=True)
    return model.eval()

def resnet34pytorchload():
    model= torchmodels.resnet34(pretrained=True)
    return model.eval()

def resnet50pytorchload():
    model= torchmodels.resnet50(pretrained=True)
    return model.eval()

def resnet101pytorchload():
    model= torchmodels.resnet101(pretrained=True)
    return model.eval()

def resnet152pytorchload():
    model= torchmodels.resnet152(pretrained=True)
    return model.eval()

def squeezenet10pytorchload():
    model= torchmodels.squeezenet1_0(pretrained=True)
    return model.eval()

def squeezenet11pytorchload():
    model= torchmodels.squeezenet1_1(pretrained=True)
    return model.eval()

def resnet34kvasirpytorchload():
    pathModel = str(
        Path.home()) + os.sep + 'DeepClas4BioModels' + os.sep + 'PyTorch' + os.sep + 'Classification' + os.sep + 'model'
    pathWeights = str(
        Path.home()) + os.sep + 'DeepClas4BioModels' + os.sep + 'PyTorch' + os.sep + 'Classification' + os.sep + 'weights'
    if not os.path.exists(pathModel + os.sep + 'ResNet34Kvasir.py'):
        if not os.path.exists(pathModel):
            os.makedirs(pathModel)

        r = requests.get('https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/Edg-3LK7P5RBggEfkOU86fIByryGl3tmsvqhuBoGsXZ91A?download=1')
        with open(pathModel + os.sep + 'ResNet34Kvasir.py', 'wb') as f:
            f.write(r.content)
    if not os.path.exists(pathWeights + os.sep + 'ResNet34Kvasir.pth'):
        if not os.path.exists(pathWeights):
            os.makedirs(pathWeights)
        r=requests.get(
            'https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/EVlYtl3fsjVLp2SjhGrZAbYB3HtYxa9PWM2y9eHI4PKIlw?download=1')
        with open(pathWeights + os.sep + 'ResNet34Kvasir.pth', 'wb') as f:
            f.write(r.content)

    return loadModel('ResNet34Kvasir')


######## METHODS FOR PREPROCESS ########
def commonPreProcess(im):
    img_pil=Image.open(im)
    prep=transforms.Compose([
        transforms.CenterCrop(224),
        transforms.ToTensor()
    ])
    img_prep=prep(img_pil)
    img_var=Variable(img_prep)
    img_var=img_var.unsqueeze(0)
    return img_var


def vgg11pytorchpreprocess(im):
    return commonPreProcess(im)

def vgg13pytorchpreprocess(im):
    return commonPreProcess(im)

def vgg16pytorchpreprocess(im):
    return commonPreProcess(im)


def vgg19pytorchpreprocess(im):
    return commonPreProcess(im)


def alexnetpytorchpreprocess(im):
    return commonPreProcess(im)

def densenet121pytorchpreprocess(im):
    return commonPreProcess(im)

def densenet161pytorchpreprocess(im):
    return commonPreProcess(im)

def densenet169pytorchpreprocess(im):
    return commonPreProcess(im)

def densenet201pytorchpreprocess(im):
    return commonPreProcess(im)

def inceptionv3pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet18pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet34pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet50pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet101pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet152pytorchpreprocess(im):
    return commonPreProcess(im)

def squeezenet10pytorchpreprocess(im):
    return commonPreProcess(im)

def squeezenet11pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet34kvasirpytorchpreprocess(im):
    return commonPreProcess(im)


######## METHODS FOR POSPROCESS ########
def commonPostProcess(result):
    path = str(Path.home()) + os.sep + 'DeepClas4BioModels'
    if not os.path.exists(path + os.sep + 'synset_words.txt'):
        if not os.path.exists(path):
            os.makedirs(path)
        r = requests.get(
            'https://unirioja-my.sharepoint.com/:t:/g/personal/adines_unirioja_es/ERS2ZWkLvc1AqY8FqIEjKBQB8MMobadwzrWsw4g86DBdAg?download=1')
        with open(path + os.sep + 'synset_words.txt', 'wb') as f:
            f.write(r.content)
    labels = np.loadtxt(path + os.sep+"synset_words.txt", str, delimiter='\n')
    return labels[result.argmax()]


def vgg11pytorchpostprocess(result):
    return commonPostProcess(result)

def vgg13pytorchpostprocess(result):
    return commonPostProcess(result)

def vgg16pytorchpostprocess(result):
    return commonPostProcess(result)

def vgg19pytorchpostprocess(result):
    return commonPostProcess(result)

def alexnetpytorchpostrocess(result):
    return commonPostProcess(result)

def densenet121pytorchpostrocess(result):
    return commonPostProcess(result)

def densenet161pytorchpostrocess(result):
    return commonPostProcess(result)

def densenet169pytorchpostrocess(result):
    return commonPostProcess(result)

def densenet201pytorchpostrocess(result):
    return commonPostProcess(result)

def inceptionv3pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet18pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet34pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet50pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet101pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet152pytorchpostprocess(result):
    return commonPostProcess(result)

def squeezenet10pytorchpostprocess(result):
    return commonPostProcess(result)

def squeezenet11pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet34kvasirpytorchpostprocess(result):
    labels = ['dyed-lifted-polyps', 'dyed-resection-margins', 'esophagitis', 'normal-cecum', 'normal-pylorus',
              'normal-z-line', 'polyps', 'ulcerative-colitis']
    return labels[result.argmax()]




class Model:

    def __init__(self,name,loadModel,preProcess,postProcess):
        self.deepModel=loadModel()
        self.preProcessor=preProcess
        self.postProcessor=postProcess
        self.name=name


import os
import numpy as np
#import sys
#sys.path.append(r'C:\Users\adines\Downloads\caffe\python')
import caffe
from pathlib import Path
import requests

# Add your model here
models=['VGG16','VGG19','AlexNet','CaffeNet','GoogleNet']

######## METHODS FOR LOAD MODELS ########

# Generic method to laod models from name
def loadModel(modelName):

    path = str(Path.home()) + os.sep + 'DeepClas4BioModels' + os.sep + 'Caffe' + os.sep

    pathModel='Classification'+os.sep+'model'+os.sep+modelName+'.prototxt'
    pathModel=path+pathModel
    pathWeights='Classification'+os.sep+'weights'+os.sep+modelName+'.caffemodel'
    pathWeights=path+pathWeights
    net=caffe.Classifier(pathModel,pathWeights,
                         channel_swap=(2,1,0),
                         raw_scale=225,
                         image_dims=(255,255))
    return net

def downloadModel(modelName,urlModel,urlWeights):
    pathModel = str(Path.home()) + os.sep + 'DeepClas4BioModels' + os.sep + 'Caffe' + os.sep + 'Classification' + os.sep + 'model'
    pathWeights = str(Path.home()) + os.sep + 'DeepClas4BioModels' + os.sep + 'Caffe' + os.sep + 'Classification' + os.sep + 'weights'
    if not os.path.exists(pathModel + os.sep + modelName+'.prototxt'):
        if not os.path.exists(pathModel):
            os.makedirs(pathModel)
        r = requests.get(urlModel)
        with open(pathModel + os.sep + modelName+'.prototxt', 'wb') as f:
            f.write(r.content)
    if not os.path.exists(pathWeights + os.sep + modelName+'.caffemodel'):
        if not os.path.exists(pathWeights):
            os.makedirs(pathWeights)
        r = requests.get(urlWeights)
        with open(pathWeights + os.sep + modelName+'.caffemodel', 'wb') as f:
            f.write(r.content)


def vgg16caffeload():
    downloadModel('VGG16','https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/EUbdJT9GgVJDsMVVrwk1vewBP-IpOfKwnJh3sSNeXdoX7g?download=1',
                  'https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/EbA3miHHnbxFrPMPB8b0QkUBFxK2r4yvq2Hk4OcU1c9ryg?download=1')
    return loadModel('VGG16')

def vgg19caffeload():
    downloadModel('VGG19','https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/EVRmVQK2uPdEk4E-xOOs88wBeLns11VEtT6QsKLztc7i7A?download=1',
                  'https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/EVUQxa-7Ya1DklQgSgKDgt8B5RqAazeyHsIKuAzhPKDGTw?download=1')
    return loadModel('VGG19')

def alexnetcaffeload():
    downloadModel('AlexNet','https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/ERYGs4x82xRNgQr4PqLUXVoB336TjhpF1BPUjj05LMp6IQ?download=1',
                  'https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/EQ_ZexbsIHtDpK5LiW0S_UoBhSgcUC7ttQuPu_DjqV39Pg?download=1')
    return loadModel('AlexNet')

def caffenetcaffeload():
    downloadModel('CaffeNet','https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/Ea8iImrYGAVNpBI1wYwhVisBYX_fWQNtdbLc228Em6Npug?download=1',
                  'https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/EVt8e-ngX1FHv07nHRLXaAcB9dnwX9i33dP8D64df150DQ?download=1')
    return loadModel('CaffeNet')

def googlenetcaffeload():
    downloadModel('GoogleNet','https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/EaNUNfz20hRLtGcWikgrRuQBwyyMyySc5sc-LVmz_TZypQ?download=1',
                  'https://unirioja-my.sharepoint.com/:u:/g/personal/adines_unirioja_es/EXGYXCCwjllPiN0DPFZJ5S8BewR7j6MrPwjuc206ONtiug?download=1')
    return loadModel('GoogleNet')


######## METHODS FOR PREPROCESS ########
def commonPreProcess(im):
    input_image=caffe.io.load_image(im)
    return input_image

def vgg16caffepreprocess(im):
    return commonPreProcess(im)

def vgg19caffepreprocess(im):
    return commonPreProcess(im)

def alexnetcaffepreprocess(im):
    return commonPreProcess(im)

def caffenetcaffepreprocess(im):
    return commonPreProcess(im)

def googlenetcaffepreprocess(im):
    return commonPreProcess(im)


######## METHODS FOR POSPROCESS ########
def commonPostProcess(result):
    path = str(Path.home()) + os.sep + 'DeepClas4BioModels'
    if not os.path.exists(path + os.sep + 'synset_words.txt'):
        if not os.path.exists(path):
            os.makedirs(path)
        r = requests.get(
            'https://unirioja-my.sharepoint.com/:t:/g/personal/adines_unirioja_es/ERS2ZWkLvc1AqY8FqIEjKBQB8MMobadwzrWsw4g86DBdAg?download=1')
        with open(path + os.sep + 'synset_words.txt', 'wb') as f:
            f.write(r.content)
    labels = np.loadtxt(path +os.sep+ "synset_words.txt", str, delimiter='\n')
    return labels[result[0].argmax()]

def vgg16caffepostprocess(result):
    return commonPostProcess(result)

def vgg19caffepostprocess(result):
    return commonPostProcess(result)

def alexnetcaffepostrocess(result):
    return commonPostProcess(result)

def caffenetcaffepostprocess(result):
    return commonPostProcess(result)

def googlenetcaffepostprocess(result):
    return commonPostProcess(result)